/*
 * main.c
 * Author: Texas Instruments
 * Date: 10/15/2020
 *
 * Description: A basic version of code to set up the TCAN114x and configure it for partial networking, then put it to sleep.
 *
 *  Pinout
 *   - P1.4 SPI Clock / SCLK
 *   - P1.6 MOSI / SDI
 *   - P1.7 MISO / SDO / nINT
 *   - P2.5 SPI Chip Select / nCS
 *
 *   - Ground wire is important
 *
 *
 *
 * Copyright (c) 2020 Texas Instruments Incorporated.  All rights reserved.
 * Software License Agreement
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Texas Instruments Incorporated nor the names of
 * its contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <driverlib.h>
#include <msp430.h>

#include "tcan114x/tcan114x.h"


// Select which device is in use to disable non-relevant features
//#define DEVICE_TCAN1144
//#define DEVICE_TCAN1145
#define DEVICE_TCAN1146


void Init_GPIO(void);
void Init_Clock(void);
void Init_SPI(void);
void Pin6_SPI_Config(void);
void Pin6_INT_Config(void);
void tcan114x_basicInitialization(void);

volatile uint8_t TCAN_Int_Cnt = 0;					// A variable used to keep track of interrupts the MCAN Interrupt pin
volatile uint8_t TCAN_go_to_sleep = 1;              // Variable used to track go-to-sleep from button press


int
main(void)
{
	/***********************************
	 * MSP430 Specific Initializations *
	 ***********************************/
	WDT_A_hold(__MSP430_BASEADDRESS_WDT_A__);
	Init_GPIO();									// Set up GPIOs for SPI and TCAN4550 connections
	Init_Clock();									// Set up the system clocks for 16 MHz (on the MSP430)
	Init_SPI();										// Initialize the SPI hardware module for 2 MHz SPI
	GPIO_clearInterrupt(GPIO_PORT_P1, GPIO_PIN1);   // Clear any interrupts on pin 1.1 before we enable global interrupts
	GPIO_clearInterrupt(GPIO_PORT_P2, GPIO_PIN3);	// Clear any interrupts on pin 2.3 before we enable global interrupts
	__enable_interrupt();


	/*********************************************
	 * Everything at this point is for TCAN114x  *
	 *********************************************/
	TCAN_Int_Cnt = 1;                               // Set int_count to 1 to service interrupts at startup

	while(1)
	{
	    if (TCAN_Int_Cnt > 0 )
	    {
	        Pin6_SPI_Config();

	        TCAN_Int_Cnt--;
	        TCAN114x_Interrupts ir = {0};                       // Define a new Device IR object for interrupt checking
	        TCAN114x_Device_Interrupts_Read(&ir);               // Read the device interrupt register
	        TCAN114x_Device_Interrupts_ClearAll();              // Clear all interrupts

	        if (ir.PWRON || ir.UVSUP || ir.UVIO ||
	            ir.UVCC  || ir.TSD   || ir.TSDW || TCAN_go_to_sleep)                // Interrupts that indicate loss of power or register data
	        {
	            tcan114x_basicInitialization();                 // Reinitialize device in case configuration was lost
	            TCAN114x_Mode_Set(TCAN114X_MODE_SLEEP);         // Put the device to sleep to test selective wake
	            TCAN_go_to_sleep = 0;                           // reset button press variable
	        }
	        if (ir.CANINT || ir.LWU)                            // CAN bus wake up or local wake up
	        {
	            TCAN114x_Mode_Set(TCAN114X_MODE_NORMAL);        // Put device to normal mode to service wake request
	        }

	        Pin6_INT_Config();
	    }
	}

}

/*
 * Configure the TCAN114x
 */
void tcan114x_basicInitialization(void)
{
    TCAN114x_Interrupts ir = {0};
    TCAN114x_Device_Interrupts_Read(&ir);
    TCAN114x_Device_Interrupts_ClearAll();

#if defined(DEVICE_TCAN1145) || defined(DEVICE_TCAN1146)
/*
 * Configure TCAN114x to wake on ID = 0x144 (standard ID)
 */
    TCAN114x_PN_Config pnConfig = {0};
    pnConfig.CAN_DR = TCAN114X_PN_CAN_500KBPS;
    pnConfig.FD_DR = TCAN114X_PN_CANFD_LESSTHAN4XCANRATE;
    pnConfig.IDE = 0;
    pnConfig.ID = 0x144;
    pnConfig.IDMask = 0x000;
    pnConfig.FrameErrorThreshold = 31;
    pnConfig.SWFDPassive = 1;
    pnConfig.DLC = TCAN114X_PN_DLC_3B;
    pnConfig.DataMaskEN = 0;
    pnConfig.DATA[0] = 0x00;
    pnConfig.DATA[1] = 0x00;
    pnConfig.DATA[2] = 0x00;
    pnConfig.DATA[3] = 0x00;
    pnConfig.DATA[4] = 0x00;
    pnConfig.DATA[5] = 0x00;
    pnConfig.DATA[6] = 0x00;
    pnConfig.DATA[7] = 0x00;

    TCAN114x_PN_Config_Write(&pnConfig);
#endif

/*
 * Configure TCAN114x mode control and enable selective wake
 */
    TCAN114x_Mode_Control mc = {0};
    TCAN114x_Mode_Control_Read(&mc);

    TCAN114x_Mode_Control modeRegister = {0};
#if defined(DEVICE_TCAN1145) || defined(DEVICE_TCAN1146)
    modeRegister.SW_EN = 1;
#else
    modeRegister.SW_EN = 0;
#endif
    modeRegister.DTO_DIS = 0;
    modeRegister.FAULT_DETECT_EN = 0;
    modeRegister.MODE_SEL = mc.MODE_SEL;
    TCAN114x_Mode_Control_Write(&modeRegister);
}

/*
 * Set TCAN114x pin 6 to be used as SDO
 */
void
Pin6_SPI_Config()
{
    //P1.7(MISO on UCB0SOMI)
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P1,
        GPIO_PIN7,
        GPIO_PRIMARY_MODULE_FUNCTION
    );
}

/*
 * Set TCAN114x pin 6 to be used as nINT
 */
void
Pin6_INT_Config()
{
    //P1.7(nINT as input with pull-up)
    GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P1, GPIO_PIN7);
    GPIO_selectInterruptEdge(GPIO_PORT_P1, GPIO_PIN7, GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_clearInterrupt(GPIO_PORT_P1, GPIO_PIN7);
    GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN7);
}

/*
 * GPIO Initialization
 */
void
Init_GPIO()
{
	// Set all GPIO pins to output low to prevent floating input and reduce power consumption
	GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P3, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P5, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P6, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P7, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P8, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P9, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);

	GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P2, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P5, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P6, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P7, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P8, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P9, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);


	// Configure P1.1 interrupt for S1 (left button on launchpad)
	GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P1, GPIO_PIN1);
	GPIO_selectInterruptEdge(GPIO_PORT_P1, GPIO_PIN1, GPIO_HIGH_TO_LOW_TRANSITION);
	GPIO_clearInterrupt(GPIO_PORT_P1, GPIO_PIN1);
	GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN1);

	// Set P4.1 and P4.2 as Secondary Module Function Input, LFXT.
	GPIO_setAsPeripheralModuleFunctionInputPin(
	        GPIO_PORT_PJ,
	        GPIO_PIN4 + GPIO_PIN5,
	        GPIO_PRIMARY_MODULE_FUNCTION
	);

	/*********************************************************
	 * 					SPI Interface Pins
	 *********************************************************/
	//P1.4(SPI CLK on UCB0CLK)
	GPIO_setAsPeripheralModuleFunctionOutputPin(
		GPIO_PORT_P1,
		GPIO_PIN4,
		GPIO_PRIMARY_MODULE_FUNCTION
	);

	//P1.6(MOSI on UCB0SIMO)
	GPIO_setAsPeripheralModuleFunctionOutputPin(
		GPIO_PORT_P1,
		GPIO_PIN6,
		GPIO_PRIMARY_MODULE_FUNCTION
	);

	//P1.7(MISO on UCB0SOMI)
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_P1,
		GPIO_PIN7,
		GPIO_PRIMARY_MODULE_FUNCTION
	);

	//set P2.5 as SPI CS, already set to output above
	GPIO_setOutputLowOnPin(GPIO_PORT_P2, GPIO_PIN5);
	GPIO_setOutputHighOnPin(GPIO_PORT_P2, GPIO_PIN5);

	// Disable the GPIO power-on default high-impedance mode
	// to activate previously configured port settings
	PMM_unlockLPM5();
}

/*
 * Clock System Initialization
 */
void Init_Clock()
{
	// Set DCO frequency to default 8MHz
	CS_setDCOFreq(CS_DCORSEL_0, CS_DCOFSEL_6);

	// Configure MCLK and SMCLK to 8MHz
	CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
	CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);

	// Initializes the XT1 crystal oscillator
	CS_turnOnLFXT(CS_LFXT_DRIVE_3);
}


/*
 * Initialize the EUSCI B SPI
 */
void Init_SPI()
{
	struct EUSCI_B_SPI_initMasterParam SPIParam = {0};

	SPIParam.selectClockSource=EUSCI_B_SPI_CLOCKSOURCE_SMCLK;
	SPIParam.clockSourceFrequency=8000000;
	SPIParam.desiredSpiClock=2000000;
	SPIParam.msbFirst=EUSCI_B_SPI_MSB_FIRST;
	SPIParam.clockPhase=EUSCI_B_SPI_PHASE_DATA_CAPTURED_ONFIRST_CHANGED_ON_NEXT;
	SPIParam.clockPolarity=EUSCI_B_SPI_CLOCKPOLARITY_INACTIVITY_LOW;
	SPIParam.spiMode=EUSCI_B_SPI_4PIN_UCxSTE_ACTIVE_HIGH;


	EUSCI_B_SPI_initMaster(EUSCI_B0_BASE, &SPIParam);
	EUSCI_B_SPI_select4PinFunctionality(EUSCI_B0_BASE,0x00);

	EUSCI_B_SPI_enable(EUSCI_B0_BASE);
}


/*
 * PORT1 Interrupt Service Routine
 */
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR(void)
{
    switch(__even_in_range(P1IV, P1IV_P1IFG7))
    {
    case P1IV_NONE : break;
    case P1IV_P1IFG0 : break;
    case P1IV_P1IFG1 :
        TCAN_go_to_sleep = 1;
        TCAN_Int_Cnt++;
    case P1IV_P1IFG2 : break;
    case P1IV_P1IFG3 : break;
    case P1IV_P1IFG4 : break;
    case P1IV_P1IFG5 : break;
    case P1IV_P1IFG6 : break;
    case P1IV_P1IFG7 : TCAN_Int_Cnt++; break;
    }
}

/*
 * PORT2 Interrupt Service Routine
 */
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR(void)
{
	switch(__even_in_range(P2IV, P2IV_P2IFG7))
	{
	case P2IV_NONE : break;
	case P2IV_P2IFG0 : break;
	case P2IV_P2IFG1 : break;
	case P2IV_P2IFG2 : break;
	case P2IV_P2IFG3 : break;
	case P2IV_P2IFG4 : break;
	case P2IV_P2IFG5 : break;
	case P2IV_P2IFG6 : break;
	case P2IV_P2IFG7 : break;
	}
}
