/*
 * tcan114x_SPI.c
 * Description: This file is responsible for abstracting the lower-level microcontroller SPI read and write functions
 *
 * Created on: December 4, 2018
 *     Author: Texas Instruments
 *
 *
 * Copyright (c) 2017 Texas Instruments Incorporated.  All rights reserved.
 * Software License Agreement
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Texas Instruments Incorporated nor the names of
 * its contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "tcan114x_spi.h"

//--------------------------------------------------------------------------
//                          Write Functions
//--------------------------------------------------------------------------
/* TCAN_WRITE
 * Description: Writes a single 8-bit word to the specified register address. Data is written from the data pointer. Function returns interrupt data
 */
uint8_t TCAN_WRITE(uint8_t address, uint8_t *data)
{

    uint8_t out = 0;
    uint8_t intRegister = 0;
#ifdef __MSP430FR6989__
    // Build the first byte to be transmitted, which is the header byte.
    out = (address << 1) | SPI_WRITE_OPCODE;

    // Set the CS low to start the transaction
    GPIO_setOutputLowOnPin(SPI_CS_GPIO_PORT, SPI_CS_GPIO_PIN);

    // Transmit the header (7-bit register address + nR/W bit)
    EUSCI_B_SPI_transmitData(SPI_HW_ADDR, out);
    WAIT_FOR_IDLE;

    // Grab the first byte coming out of the device during the header byte. This is the interrupt register.
    intRegister = HWREG8(SPI_HW_ADDR + OFS_UCBxRXBUF);

    // Send the data byte
    EUSCI_B_SPI_transmitData(SPI_HW_ADDR, *data);
    WAIT_FOR_IDLE;

    // Pull CS pin high
    GPIO_setOutputHighOnPin(SPI_CS_GPIO_PORT, SPI_CS_GPIO_PIN);
#endif
    return intRegister;
}

/* TCAN_WRITE_BURST
 * Description: Writes multiple 8-bit words to the specified register address. Data is written from the data array. Function returns interrupt data
 */
uint8_t TCAN_WRITE_BURST(uint8_t address, uint8_t numBytes, uint8_t data[])
{

    uint32_t out = 0;
    uint8_t intRegister = 0;
    uint8_t bytesToTransfer = numBytes;

    // Burst writes can only be up to 3 bytes, so this is some basic validation
    if (numBytes > 3)
        bytesToTransfer = 3;

#ifdef __MSP430FR6989__
    // Build the first byte to be transmitted, which is the header byte.
    out = (address << 1) | SPI_WRITE_OPCODE;

    // Set the CS low to start the transaction
    GPIO_setOutputLowOnPin(SPI_CS_GPIO_PORT, SPI_CS_GPIO_PIN);

    // Transmit the header (7-bit register address + nR/W bit)
    EUSCI_B_SPI_transmitData(SPI_HW_ADDR, out);
    WAIT_FOR_IDLE;

    // Grab the first byte coming out of the device during the header byte. This is the interrupt register.
    intRegister = HWREG8(SPI_HW_ADDR + OFS_UCBxRXBUF);

    // Send the data bytes
    uint8_t i;
    for (i = 0; i < bytesToTransfer; i++)
    {
        EUSCI_B_SPI_transmitData(SPI_HW_ADDR, data[i]);
        WAIT_FOR_TRANSMIT;
    }
    WAIT_FOR_IDLE;

    // Pull CS pin high
    GPIO_setOutputHighOnPin(SPI_CS_GPIO_PORT, SPI_CS_GPIO_PIN);
#endif
    return intRegister;
}

//--------------------------------------------------------------------------
//                          Read Functions
//--------------------------------------------------------------------------
/* TCAN_READ
 * Description: Reads a single 8-bit word to the specified register address. Data is written to data pointer. Function returns interrupt data
 */
uint8_t TCAN_READ(uint8_t address, uint8_t *data)
{
    uint32_t out = 0;
    uint8_t intRegister = 0;

#ifdef __MSP430FR6989__
    // Build the first byte to be transmitted, which is the header byte.
    out = (address << 1) | SPI_READ_OPCODE;

    // Set the CS low to start the transaction
    GPIO_setOutputLowOnPin(SPI_CS_GPIO_PORT, SPI_CS_GPIO_PIN);

    // Transmit the header (7-bit register address + nR/W bit)
    EUSCI_B_SPI_transmitData(SPI_HW_ADDR, out);
    WAIT_FOR_IDLE;

    // Grab the first byte coming out of the device during the header byte. This is the interrupt register.
    intRegister = HWREG8(SPI_HW_ADDR + OFS_UCBxRXBUF);

    // Send 0x00 just to get data out of the device
    EUSCI_B_SPI_transmitData(SPI_HW_ADDR, 0x00);
    WAIT_FOR_IDLE;
    *data = HWREG8(SPI_HW_ADDR + OFS_UCBxRXBUF);

    // Pull CS pin high
    GPIO_setOutputHighOnPin(SPI_CS_GPIO_PORT, SPI_CS_GPIO_PIN);
#endif
    return intRegister;

}

/* TCAN_READ_BURST
 * Description: Reads multiple 8-bit words from the specified register address. Data read is stored to data pointer. Function returns interrupt data
 */
uint8_t TCAN_READ_BURST(uint8_t address, uint8_t numBytes, uint8_t data[])
{
    uint32_t out = 0;
    uint8_t intRegister = 0;
    uint8_t bytesToTransfer = numBytes;

    // Burst writes can only be up to 3 bytes, so this is some basic validation
    if (numBytes > 3)
        bytesToTransfer = 3;


#ifdef __MSP430FR6989__
    // Build the first byte to be transmitted, which is the header byte.
    out = (address << 1) | SPI_READ_OPCODE;

    // Set the CS low to start the transaction
    GPIO_setOutputLowOnPin(SPI_CS_GPIO_PORT, SPI_CS_GPIO_PIN);

    // Transmit the header (7-bit register address + nR/W bit)
    EUSCI_B_SPI_transmitData(SPI_HW_ADDR, out);
    WAIT_FOR_IDLE;

    // Grab the first byte coming out of the device during the header byte. This is the interrupt register.
    intRegister = HWREG8(SPI_HW_ADDR + OFS_UCBxRXBUF);

    // Send 0x00 just to get data out of the device
    uint8_t i;
    for (i = 0; i < bytesToTransfer; i++)
    {

        EUSCI_B_SPI_transmitData(SPI_HW_ADDR, 0x00);
        WAIT_FOR_IDLE;
        data[i] = HWREG8(SPI_HW_ADDR + OFS_UCBxRXBUF);
    }

    // Pull CS pin high
    GPIO_setOutputHighOnPin(SPI_CS_GPIO_PORT, SPI_CS_GPIO_PIN);
#endif
    return intRegister;

}
